""" Very simple client implementation to use ETI TAXII server. Using cabby and stix::
    pip install cabby
    pip install stix2

Compatible also with old Python 2.7 (see code lines with comment 'Compatibility with Python 2.7').

To run this test, you need to create also 'test_credentials.txt' which will contain 3 lines:
    host    (should be: "eti.eset.com")
    username
    password
"""

import datetime
import json
import os
import sys  # Used only to check compatibility with Python 2.7.

# 3rd party packages
import cabby
import stix2

if sys.version_info.major == 2:  # Compatibility with Python 2.7.
    import pytz

#CA_CERT = os.path.join(os.path.dirname(__file__), "ca_cert.crt")
#if not os.path.isfile(CA_CERT):
#    CA_CERT = None

def main():
    """ Download all available packages from last hour and print them.
    """
    with open("test_credentials.txt") as file:
        host = file.readline().strip()
        username = file.readline().strip()
        password = file.readline().strip()
    client = cabby.create_client(host, discovery_path="/taxiiservice/discovery", use_https=True)
    client.set_auth(username=username, password=password)
    #client.set_auth(username=username, password=password, ca_cert=CA_CERT)
    # 1. Get all available collections.
    collections = [c for c in client.get_collections() if c.available]
    # 2. Download data from last few hours and check if they have valid format.
    if sys.version_info.major == 2:  # Compatibility with Python 2.7.
        end_date = datetime.datetime.now(pytz.utc)
    else:
        end_date = datetime.datetime.now(datetime.timezone.utc)
    begin_date = end_date - datetime.timedelta(hours=1)
    for col in collections:
        base_dir = os.path.join(".", col.name)
        if sys.version_info.major == 2:  # Compatibility with Python 2.7.
            if not os.path.isdir(base_dir):
                os.makedirs(base_dir)
        else:
            os.makedirs(base_dir, exist_ok=True)
        i = 0
        for item in client.poll(col.name, begin_date=begin_date, end_date=end_date):
            if not item.content: # Skip empty packages.
                continue
            i += 1
            if i > 10:  # Download only a few packages - as an example.
                break
            file_path = os.path.join(base_dir, item.timestamp.strftime("{0:02d} %Y-%m-%d %H-%M.data".format(i)))
            with open(file_path, "wb") as file:
                file.write(item.content)
            parsed = None
            if col.content_bindings[0].id.startswith("urn:stix.mitre.org:json:2.0"):
                with open(file_path) as file:
                    parsed = stix2.parse(file)
            elif col.content_bindings[0].id.startswith("urn:eset.com:json"):
                with open(file_path) as file:
                    parsed = json.load(file)
            print("{}: {}".format(file_path, str(parsed)[0:100])) # Print only first 100 bytes of parsed data.


if __name__ == "__main__":
    main()
